/*
 * Copyright 2002, Tobias Huerlimann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#define INCL_DOSERRORS
#define INCL_DOSQUEUES
#define INCL_DOSPROCESS
#include <os2.h>

#include "pmmpgp.h"

#define MSG_GOODSIG		0
#define MSG_EXPSIG		1
#define MSG_EXPKEYSIG		2
#define MSG_BADSIG		3
#define MSG_ERRSIG		4
#define MSG_VALIDSIG		5
#define MSG_SIG_ID		6
#define MSG_ENC_TO		7
#define MSG_NODATA		8
#define MSG_UNEXPECTED		9
#define MSG_TRUST_UNDEFINED	10
#define MSG_TRUST_NEVER		11
#define MSG_TRUST_MARGINAL	12
#define MSG_TRUST_FULLY		13
#define MSG_TRUST_ULTIMATE	14
#define MSG_SIGEXPIRED		15
#define MSG_KEYEXPIRED		16
#define MSG_KEYREVOKED		17
#define MSG_BADARMOR		18
#define MSG_RSA_OR_IDEA		19
#define MSG_SHM_INFO		20
#define MSG_SHM_GET		21
#define MSG_SHM_GET_BOOL	22
#define MSG_SHM_GET_HIDDEN	23
#define MSG_GET_BOOL		24
#define MSG_GET_LINE		25
#define MSG_GET_HIDDEN		26
#define MSG_GOT_IT		27
#define MSG_NEED_PASSPHRASE	28
#define MSG_NEED_PASSPHRASE_SYM	29
#define MSG_MISSING_PASSPHRASE	30
#define MSG_BAD_PASSPHRASE	31
#define MSG_GOOD_PASSPHRASE	32
#define MSG_DECRYPTION_FAILED	33
#define MSG_DECRYPTION_OKAY	34
#define MSG_NO_PUBKEY		35
#define MSG_NO_SECKEY		36
#define MSG_IMPORTED		37
#define MSG_IMPORT_OK		38
#define MSG_IMPORT_PROBLEM	39
#define MSG_IMPORT_RES		40
#define MSG_FILE_START		41
#define MSG_FILE_DONE		42
#define MSG_BEGIN_DECRYPTION	43
#define MSG_END_DECRYPTION	44
#define MSG_BEGIN_ENCRYPTION	45
#define MSG_END_ENCRYPTION	46
#define MSG_DELETE_PROBLEM	47
#define MSG_PROGRESS		48
#define MSG_SIG_CREATED		49
#define MSG_KEY_CREATED		50
#define MSG_SESSION_KEY		51
#define MSG_NOTATION_NAME	52
#define MSG_NOTATION_DATA	53
#define MSG_USERID_HINT		54
#define MSG_POLICY_URL		55
#define MSG_BEGIN_STREAM	56
#define MSG_END_STREAM		57
#define MSG_INV_RECP		58
#define MSG_NO_RECP		59
#define MSG_ALREADY_SIGNED	60
#define MSG_TRUNCATED		61
#define MSG_ERROR		62
#define MSG_ATTRIBUTE		63
#define MSG_GOODMDC		64
#define MSG_NOMSG		65

typedef struct _status_msgs {
	char	name[PMMPGP_MAXNAMELEN + 1];
	char	args;
} status_msgs;

status_msgs	msgs[MSG_NOMSG] = {
	{"GOODSIG",		2},
	{"EXPSIG",		2},
	{"EXPKEYSIG",		2},
	{"BADSIG",		2},
	{"ERRSIG",		6},
	{"VALIDSIG",		4},
	{"SIG_ID",		3},
	{"ENC_TO",		3},
	{"NODATA",		1},
	{"UNEXPECTED",		1},
	{"TRUST_UNDEFINED",	1},
	{"TRUST_NEVER",		1},
	{"TRUST_MARGINAL",	0},
	{"TRUST_FULLY",		0},
	{"TRUST_ULTIMATE",	0},
	{"SIGEXPIRED",		0},
	{"KEYEXPIRED",		1},
	{"KEYREVOKED",		0},
	{"BADARMOR",		0},
	{"RSA_OR_IDEA",		0},
	{"SHM_INFO",		0},
	{"SHM_GET",		0},
	{"SHM_GET_BOOL",	0},
	{"SHM_GET_HIDDEN",	0},
	{"GET_BOOL",		0},
	{"GET_LINE",		0},
	{"GET_HIDDEN",		0},
	{"GOT_IT",		0},
	{"NEED_PASSPHRASE",	4},
	{"NEED_PASSPHRASE_SYM",	3},
	{"MISSING_PASSPHRASE",	0},
	{"BAD_PASSPHRASE",	1},
	{"GOOD_PASSPHRASE",	0},
	{"DECRYPTION_FAILED",	0},
	{"DECRYPTION_OKAY",	0},
	{"NO_PUBKEY",		1},
	{"NO_SECKEY",		1},
	{"IMPORTED",		2},
	{"IMPORT_OK",		2},
	{"IMPORT_PROBLEM",	2},
	{"IMPORT_RES",		13},
	{"FILE_START",		2},
	{"FILE_DONE",		0},
	{"BEGIN_DECRYPTION",	0},
	{"END_DECRYPTION",	0},
	{"BEGIN_ENCRYPTION",	2},
	{"END_ENCRYPTION",	0},
	{"DELETE_PROBLEM",	1},
	{"PROGRESS",		4},
	{"SIG_CREATED",		6},
	{"KEY_CREATED",		1},
	{"SESSION_KEY",		1},
	{"NOTATION_NAME",	1},
	{"NOTATION_DATA",	1},
	{"USERID_HINT",		2},
	{"POLICY_URL",		1},
	{"BEGIN_STREAM",	0},
	{"END_STREAM",		0},
	{"INV_RECP",		2},
	{"NO_RECP",		1},
	{"ALREADY_SIGNED",	1},
	{"TRUNCATED",		1},
	{"ERROR",		2},
	{"ATTRIBUTE",		8},
	{"GOODMDC",		0} };

char*		arg[PMMPGP_MAXSTATUSARGS];

int lookup_msg(const char*);

int pmmpgp_gpgstatusline(const char* my_line) {
	APIRET		rc;
	ULONG		ulWritten;
	int		i;
	char		*event, *args;
	int		n;
	time_t 		key_creation_date;
	char		key_creation_date_str[30];

	event = (char*)(my_line + 9);	/* Omit "[GNUPG:] " */
	args = strchr(event, ' ');
	if (args != NULL)
		*args++ = '\0';

	if (my_opts.verbose >= 2)
		fprintf(my_handles.log, "pmmpgp: [GPG]: %s %s\n", event, (args == NULL ? "" : args));

	n = lookup_msg(event);
	if (n == -1)
		return -1;

	if (args != NULL) {
		arg[0] = args;
		for (i = 1; i < msgs[n].args; i++) {
			arg[i] = strchr(arg[i - 1], ' ');
			if (arg[i] != NULL) {
				*arg[i]++ = '\0';
				/* In case the last character in the string is a blank */
				if (*arg[i] == '\0') {
					arg[i] = NULL;
					break;
				}
			} else
				break;
		}
	} else
		bzero(&arg, sizeof(arg));

	if (arg[msgs[n].args - 1] == NULL)
		return -1;

	switch (n) {
		case MSG_GOODSIG:
			arg[0] += 8;	/* Skip the first 8 chars to get the short key id */
			fprintf(my_handles.io.err, "Good signature from user \"%s [Key ID: %s]\".\n", arg[1], arg[0]);
			break;
		case MSG_BADSIG:
			arg[0] += 8;	/* Skip the first 8 chars to get the short key id */
			fprintf(my_handles.io.err, "WARNING: Bad signature, doesn't match file contents!\n");
			fprintf(my_handles.io.err, "Bad signature from user \"%s [Key ID: %s]\".\n", arg[1], arg[0]);
			break;
		case MSG_VALIDSIG:
			key_creation_date = atoi(arg[2]);
			if (key_creation_date == 0)
				return -1;
			strftime(key_creation_date_str, sizeof(key_creation_date_str),
					"%Y/%m/%d %H:%M GMT", gmtime(&key_creation_date));
			fprintf(my_handles.io.err, "Signature made %s\n", key_creation_date_str);
			break;
		case MSG_ERRSIG:
			fprintf(my_handles.io.err, "WARNING: Can't find the right public key-- can't check signature integrity\n");
			break;
		case MSG_NO_PUBKEY:
			arg[0] += 8;	/* Skip the first 8 chars to get the short key id */
			fprintf(my_handles.io.err, "Key matching expected Key ID %s not found in public keyring.\n", arg[0]);
			break;
		case MSG_NO_SECKEY:
			arg[0] += 8;	/* Skip the first 8 chars to get the short key id */
			fprintf(my_handles.io.err, "Key matching expected Key ID %s not found in secret keyring.\n", arg[0]);
			break;
		case MSG_INV_RECP:
			fprintf(my_handles.io.err, "Key matching userid '%s' not found in keyring.\n\n", arg[1]);
			fprintf(my_handles.io.err, "Cannot find the public key matching userid '%s'\n", arg[1]);
			fprintf(my_handles.io.err, "This user will not be able to decrypt this message.\n");
			fprintf(my_handles.io.err, "Encryption error\n");
			break;
		case MSG_BAD_PASSPHRASE:
			fprintf(my_handles.io.err, "Error:  Bad pass phrase.\n");
			fprintf(my_handles.io.err, "Signature error\n");
			gpg_cmdline.rc = 20;
			break;
	}

	return 0;
}

int lookup_msg(const char* event) {
	int		i = 0, j;
	int		looks_good, found = FALSE;

	while (!found) {
		j = 0;
		looks_good = TRUE;
		while ((event[j] != '\0') && (msgs[i].name[j] != '\0')) {
			if (event[j] != msgs[i].name[j]) {
				looks_good = FALSE;
				break;
			}
			j++;
		}
		/*
		 * Just to make sure that event "abc" isn't identified
		 * as msg "abcd"
		 */
		if (event[j] != msgs[i].name[j])
			looks_good = FALSE;
		if (looks_good)
			found = TRUE;
		else
			i++;
		if (i == MSG_NOMSG)
			break;
	}

	return (found ? i : -1);
}

