/*
 * Copyright 2002, Tobias Huerlimann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __PMMPGP_H__
#define __PMMPGP_H__

#ifdef __IBMC__
	#define bzero(a, b)	memset(a, 0, b);
#endif

#define PMMPGP_CONFIGFILE	"pmmpgp.conf"

#define PMMPGP_PIPESIZE		256
#define PMMPGP_MAXPATHLEN	CCHMAXPATH
#define PMMPGP_MAXLINELEN	1024
#define PMMPGP_MAXNAMELEN	20
#define PMMPGP_MAXVALUELEN	128
#define PMMPGP_MAXSTATUSARGS	12

typedef struct _pgp_commands {
	char		help;
	char		conventional_encrypt;
	char		decrypt;
	char		encrypt;
	char		key_help;
	char		key_add;
	char		key_check;
	char		key_disable;
	char		key_edit;
	char		key_gen;
	char		key_maint;
	char		key_remove;
	char		key_sign;
	char		key_display;
	char		key_display_fingerprint;
	char		key_display_more;
	char		key_extract;
	char		sign;
	char		wipe;
} pgp_commands;

typedef struct _pgp_options {
	char		armor;
	int		armorline;
	char*		bakring;
	char		batchmode;
	int		cert_depth;
	char*		charset;
	char		clearsig;
	char*		comment;
	int		completes_needed;
	char		compress;
	char		encrypttoself;
	char		force;
	char		interactive;
	char		keepbinary;
	char*		language;
	int		legal_kludge;
	char		makerandom;
	int		marginals_needed;
	char*		myname;
	char		nomanual;
	char*		pager;
	char*		pubring;
	char*		randseed;
	char*		secring;
	char		showpass;
	char		textmode;
	char*		tmp;
	int		tzfix;
	int		verbose;

	char		filter;
	char		seperate_signature;
	char		foryoureyesonly;
	char		outputfile[PMMPGP_MAXVALUELEN + 1];
	char		passphrase[PMMPGP_MAXVALUELEN + 1];
	char		preserve_filename;
	char		userid[PMMPGP_MAXVALUELEN + 1];
} pgp_options;

typedef struct _handles {
	struct {
		FILE	*in, *out, *err;
	}		io;
	struct {
		HPIPE	read, write;
	}		pass;
	struct {
		HPIPE	read, write;
	}		status;
	HFILE		null;
	FILE*		log;
} handles;

typedef struct _options {
	int		verbose;
	char		logfile[PMMPGP_MAXPATHLEN + 1];
	char		gpg_executable[PMMPGP_MAXPATHLEN + 1];
} options;

typedef struct _gpg_call {
	char		executable[PMMPGP_MAXPATHLEN + 1];
	char		opts[PMMPGP_MAXLINELEN + 1];
	char		rcpts[PMMPGP_MAXLINELEN + 1];
	char		cmds[PMMPGP_MAXLINELEN + 1];
	char		files[PMMPGP_MAXLINELEN + 1];
	char		env[PMMPGP_MAXLINELEN + 1];
	int		rc;
} gpg_call;

extern handles		my_handles;
extern options		my_opts;
extern pgp_options	pgp_opts;
extern pgp_commands	pgp_cmds;
extern gpg_call		gpg_cmdline;

/* From OpenBSD's libc */
extern size_t strlcat(char*, const char*, size_t);
extern size_t strlcpy(char*, const char*, size_t);

extern int pmmpgp_parse_pgp_cmdline(const int, char**);
extern int pmmpgp_gpgstatusline(const char*);

#endif
